package kd.hrmp.hbjm.business.domain.repository;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QFilter;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;
import kd.hrmp.hbjm.common.constants.ChangeEventConstants;

import java.util.List;
import java.util.Set;

public class HBJMEventRepository {
    private static class HBJMEventInstance {
        private static HBJMEventRepository INSTANCE = new HBJMEventRepository();
    }

    public static HBJMEventRepository getInstance() {
        return HBJMEventInstance.INSTANCE;
    }

    /**
     * 批量保存
     *
     * @param entityPage     元数据标识
     * @param dynamicObjects 要保存的数据
     */
    public void saveBatch(String entityPage, List<DynamicObject> dynamicObjects) {
        HRBaseServiceHelper serviceHelper = new HRBaseServiceHelper(entityPage);
        serviceHelper.save(dynamicObjects.toArray(new DynamicObject[0]));
    }

    /**
     * 根据事务id查询变动操作明细
     *
     * @param eventId
     */
    public DynamicObject[] selectOpDetail(long eventId) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(
                ChangeEventConstants.PAGE_OP_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.equals, eventId)};
        return eventsumHelper.loadDynamicObjectArray(qFilters);
    }

    /**
     * 根据事务id集合查询变动操作明细
     *
     * @param eventIds
     */
    public DynamicObject[] selectEventDetails(Set<Long>  eventIds) {
        HRBaseServiceHelper eventHelper = new HRBaseServiceHelper(
                ChangeEventConstants.PAGE_EVENT_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.in, eventIds)};
        return eventHelper.loadDynamicObjectArray(qFilters);
    }

    /**
     * 根据事务id集合查询变动事务汇总
     *
     * @param eventIds
     */
    public DynamicObject[] selectEventSumDetails(Set<Long> eventIds) {
        HRBaseServiceHelper eventSumHelper = new HRBaseServiceHelper(
                ChangeEventConstants.PAGE_EVENT_SUM);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.in, eventIds)};
        return eventSumHelper.loadDynamicObjectArray(qFilters);
    }

    /**
     * 根据事务id删除事务汇总
     *
     * @param eventIds 事务id
     */
    public void delEventSum(Set<Long> eventIds) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_EVENT_SUM);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.in, eventIds)};
        eventsumHelper.deleteByFilter(qFilters);
    }

    /**
     * 根据事务id删除事务明细
     *
     * @param eventIds
     */
    public void delEventDetail(Set<Long> eventIds) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_EVENT_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.in, eventIds)};
        eventsumHelper.deleteByFilter(qFilters);
    }

    /**
     * 根据事务id删除变动操作明细
     *
     * @param eventIds
     */
    public void delOpDetail(Set<Long> eventIds) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_OP_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("event", QFilter.in, eventIds)};
        eventsumHelper.deleteByFilter(qFilters);
    }

    /**
     * 根据boids查询操作明细明细
     *
     * @param boids
     */
    public DynamicObject[] selectOpDetailByBoIds(Set<Long> boids) {
        HRBaseServiceHelper eventsumHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_OP_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("job", QFilter.in, boids)};
        return eventsumHelper.loadDynamicObjectArray(qFilters);
    }

    /**
     * 查询所有操作明细明细
     *
     */
    public DynamicObject[] selectAllOpDetail() {
        HRBaseServiceHelper changeOperationDetailHelper = new HRBaseServiceHelper(ChangeEventConstants.PAGE_OP_DETAIL);
        QFilter[] qFilters = new QFilter[]{new QFilter("1", QFilter.in, 1)};
        return changeOperationDetailHelper.loadDynamicObjectArray(qFilters);
    }

}
