package kd.hrmp.hbjm.business.domain.service.impl;

import kd.bos.dataentity.Consumer;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.ILocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.property.BasedataProp;
import kd.bos.entity.property.MuliLangTextProp;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.hr.hbp.common.constants.HRBaseConstants;
import kd.hr.hbp.common.util.HRStringUtils;
import kd.hrmp.hbjm.business.domain.service.IBaseValidatorService;

import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author rd_ningjie
 * @description:TODO 职位基础校验
 * @date 2022/5/30 11:03
 */
public class JobAbstractBaseValidatorService implements IBaseValidatorService {

    private static Log LOG = LogFactory.getLog(JobAbstractBaseValidatorService.class);

    private static class JobAbstractBaseValidatorRepository {
        private static JobAbstractBaseValidatorService INSTANCE = new JobAbstractBaseValidatorService();
    }

    public static JobAbstractBaseValidatorService getInstance() {
        return JobAbstractBaseValidatorService.JobAbstractBaseValidatorRepository.INSTANCE;
    }


    /**
     * @param datas
     * @param consumer
     * @description: 基础资料字段数据启用性校验
     * @return: void
     * @author rd_ningjie
     * @date: 2022/5/30 11:38
     */
    @Override
    public <T> void enableValide(ExtendedDataEntity[] datas, String col, Consumer<T> consumer) {

        if (ObjectUtils.isEmpty(datas) || ObjectUtils.isEmpty(col)) {
            return;
        }
        //校验实体中是否有该属性
        checkBasedataColAccess(datas[0].getDataEntity(), col);
        Arrays.stream(datas).forEach(data -> {
            DynamicObject dynamicObject = data.getDataEntity().getDynamicObject(col);
            if (!ObjectUtils.isEmpty(dynamicObject)) {
                //查询基础资料字段启用状态
                String enable = String.valueOf(dynamicObject.get(HRBaseConstants.ENABLE));
                if (!HRBaseConstants.ENABLED.equals(enable)) {
                    consumer.accept((T) data);
                }
            }
        });
    }


    /**
     * @param newDatas
     * @param sourceDatas
     * @param col
     * @param consumer
     * @description: 校验不能变更的字段
     * @return: void
     * @author rd_ningjie
     * @date: 2022/5/31 9:49
     */
    @Override
    public <T> void changeAllowAccessValde(ExtendedDataEntity[] newDatas, DynamicObject[] sourceDatas, String col, Consumer<T> consumer) {

        if (ObjectUtils.isEmpty(newDatas) || ObjectUtils.isEmpty(sourceDatas) || ObjectUtils.isEmpty(col)) {
            return;
        }
        //分别校验新数据与源数据的实体类型
        IDataEntityType newType = checkBasedataColAccess(newDatas[0].getDataEntity(), col);
        IDataEntityType sourceType = checkBasedataColAccess(sourceDatas[0], col);

        if (!newType.getName().equals(sourceType.getName())) {
            LOG.error(String.format("connt compare diffrent type of DynamicObject :% %", newType, sourceType));
            throw new RuntimeException("connt compare diffrent type of DynamicObject");
        }
        //分别对两个集合进行主键映射为map
        String primaryKey = newType.getPrimaryKey().getName();

        Map<Object, ExtendedDataEntity> newMap = Arrays.stream(newDatas).collect(Collectors.toMap(dy -> dy.getValue(primaryKey), dy -> dy));
        Map<Object, DynamicObject> sourceMap = Arrays.stream(sourceDatas).collect(Collectors.toMap(DynamicObject::getPkValue, dy -> dy));

        for (Map.Entry<Object, ExtendedDataEntity> entry : newMap.entrySet()) {
            Object key = entry.getKey();
            DynamicObject srouceDy = sourceMap.get(key);
            if (!ObjectUtils.isEmpty(srouceDy)) {
                if (compareDynamicCol(entry.getValue().getDataEntity(), srouceDy, newType, col))
                    consumer.accept((T) entry.getValue());
            }

        }
    }

    /**
     * @param datas
     * @description: 定制化比较高，子类重写
     * @return: void
     * @author rd_ningjie
     * @date: 2022/5/31 9:50
     */
    @Override
    public void availabilityValide(ExtendedDataEntity[] datas) {
        LOG.error("JobAbstractBaseValidator availabilityValide need be override");
        throw new RuntimeException("JobAbstractBaseValidator availabilityValide need be override");
    }

    /**
     * @param dynamicObject
     * @param col
     * @description: 判断动态对象中是否包含此字段并且是基础资料类型字段
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 14:07
     */
    protected IDataEntityType checkBasedataColAccess(DynamicObject dynamicObject, String col) {
        if (!ObjectUtils.isEmpty(dynamicObject) && !ObjectUtils.isEmpty(col)) {
            IDataEntityType type = dynamicObject.getDataEntityType();
            DataEntityPropertyCollection cols = type.getProperties();
            if (!ObjectUtils.isEmpty(cols) && (cols.stream().anyMatch(info -> col.equals(info.getName())))) {
                return type;
            }
        }
        LOG.error(String.format("JobAbstractBaseValidator checkBasedataColAccess DynamicObject donnot contains this col or col is not BasedataProp :% %", dynamicObject, col));
        throw new RuntimeException("DynamicObject donnot contains this col or col is not BasedataProp ");
    }

    protected IDataEntityProperty getProp(IDataEntityType type, String col) {
        if (!ObjectUtils.isEmpty(type) && !ObjectUtils.isEmpty(col)) {
            DataEntityPropertyCollection props = type.getProperties();
            for (IDataEntityProperty prop : props) {
                if (col.equals(prop.getName())) {
                    return prop;
                }
            }
        }
        LOG.error(String.format("JobAbstractBaseValidator getProp IDataEntityType donnot contains this col:% %", type, col));
        throw new RuntimeException("IDataEntityType donnot contains this col");
    }


    /**
     * @param dynamicObject
     * @param sourceDynamic
     * @param type
     * @param col
     * @description: 判断该字段是否被修改 修改了返回true 没有修改则返回false
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 16:49
     */
    public boolean compareDynamicCol(DynamicObject dynamicObject, DynamicObject sourceDynamic, IDataEntityType type, String col) {

        IDataEntityProperty prop = getProp(type, col);

        //多语言字段处理 分别对比对应语言下的信息是否被改变
        if (prop instanceof MuliLangTextProp) {
            return !compareMulLangTextCol(dynamicObject.getLocaleString(col), sourceDynamic.getLocaleString(col));
        }

        if (prop instanceof BasedataProp) {
            return !compareBaseDataCol(dynamicObject.getDynamicObject(col), sourceDynamic.getDynamicObject(col));
        }


        //分录字段 分别对比其中的基础资料id是否被改变
        if (prop instanceof DynamicObjectCollection) {
            return !compareDynaCollectCol(dynamicObject.getDynamicObjectCollection(col), sourceDynamic.getDynamicObjectCollection(col));
        }
        //普通字段，String.valueOf进行比对就行了
        return !compareCommonCol(dynamicObject.get(col), sourceDynamic.get(col));
    }

    /**
     * @param val
     * @param sourceVal
     * @description: 对比多语言字段信息, 看此字段是否被修改
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 16:38
     */
    protected boolean compareMulLangTextCol(ILocaleString val, ILocaleString sourceVal) {

        //如果同时都是空，则返回true
        if (ObjectUtils.isEmpty(val) && ObjectUtils.isEmpty(sourceVal)) return true;
        //如果不同时为空，有一个为空，则证明修改过，直接返回false
        if (ObjectUtils.isEmpty(val) || ObjectUtils.isEmpty(sourceVal)) return false;

        //如果同时都是空，则返回true
        for (Map.Entry<String, String> entry : val.entrySet()) {
            String key = entry.getKey();
            String sourceSubVal = sourceVal.get(key);
            if (!entry.getValue().equals(sourceSubVal)) {
                //有一种语言不同则返回false
                return false;
            }
        }
        return true;
    }

    /**
     * @param dynamicObject
     * @param sourceDynamic
     * @description: 判断基础资料是否被改变
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 18:42
     */
    protected boolean compareBaseDataCol(DynamicObject dynamicObject, DynamicObject sourceDynamic) {
        //如果同时都是空，则返回true
        if (ObjectUtils.isEmpty(dynamicObject) && ObjectUtils.isEmpty(sourceDynamic)) return true;
        //如果不同时为空，有一个为空，则证明修改过，直接返回false
        if (ObjectUtils.isEmpty(dynamicObject) || ObjectUtils.isEmpty(sourceDynamic)) return false;

        //对比基础资料中的id是否相同
        return HRStringUtils.equals(String.valueOf(dynamicObject.getPkValue()), String.valueOf(sourceDynamic.getPkValue()));
    }

    /**
     * @param datas
     * @param sourceDatas
     * @description: 判断分录是否被修改
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 18:48
     */
    protected boolean compareDynaCollectCol(DynamicObjectCollection datas, DynamicObjectCollection sourceDatas) {

        //如果同时都是空，则返回true
        if (ObjectUtils.isEmpty(datas) && ObjectUtils.isEmpty(sourceDatas)) return true;
        //如果不同时为空，有一个为空，则证明修改过，直接返回false
        if (ObjectUtils.isEmpty(datas) || ObjectUtils.isEmpty(sourceDatas)) return false;

        //如果size相同，则对比具体内容的id
        Map<Object, DynamicObject> dataMap = datas.stream().collect(Collectors.toMap(DynamicObject::getPkValue, dy -> dy));

        Map<Object, DynamicObject> sourceMap = sourceDatas.stream().collect(Collectors.toMap(DynamicObject::getPkValue, dy -> dy));

        //如果集合size不同，则直接返回false
        if (dataMap.size() != sourceMap.size()) return false;

        //对比分录的id
        for (Map.Entry<Object, DynamicObject> entry : dataMap.entrySet()) {
            DynamicObject srouceDy = sourceMap.get(entry.getKey());
            if (ObjectUtils.isEmpty(srouceDy)) return false;
        }
        return true;

    }

    /**
     * @param data
     * @param sourceData
     * @description: 对比普通字段
     * @return: boolean
     * @author rd_ningjie
     * @date: 2022/5/30 19:01
     */
    protected boolean compareCommonCol(Object data, Object sourceData) {
        //如果同时都是空，则返回true
        if (ObjectUtils.isEmpty(data) && ObjectUtils.isEmpty(sourceData)) return true;
        //如果不同时为空，有一个为空，则证明修改过，直接返回false
        if (ObjectUtils.isEmpty(data) || ObjectUtils.isEmpty(sourceData)) return false;

        //对比普通字段string值是否相同
        return HRStringUtils.equals(String.valueOf(data), String.valueOf(sourceData));
    }

}
