package kd.hrmp.hbjm.business.utils;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.OrmLocaleValue;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.property.*;
import kd.bos.service.InteTimeZone;
import kd.hrmp.hbjm.common.util.GroupKey;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author rd_ningjie
 * @description:组合校验构建
 * @date 2022/9/16 10:18
 */
public class GroupKeyBuilder {

    static final String GROUPKEYBUILDER_1 = "GroupKeyBuilder_1";
    static final String HRMP_HBJM_BUSINESS = "hrmp-hbjm-business";

    private List<String> propNames;

    private ExtendedDataEntity[] dataEntity;

    /**
     * @description: 私有化无参构造器
     * @return:
     * @author rd_ningjie
     * @date: 2022/9/19 14:33
     */
    private GroupKeyBuilder() {
    }

    /**
     * @param dataEntity
     * @param propNames
     * @description: 构造器 需要传入实体与字段标识，字段标识如果是联合唯一，则往里面添加多个
     * @return: kd.hrmp.hbpm.business.utils.HbpmGroupKeyBuilder
     * @author rd_ningjie
     * @date: 2022/9/19 14:34
     */
    public static GroupKeyBuilder newInstance(ExtendedDataEntity[] dataEntity, List<String> propNames) {
        GroupKeyBuilder build = new GroupKeyBuilder();
        build.dataEntity = dataEntity;
        build.propNames = propNames;
        return build;
    }

    /**
     * @description: 构建key
     * @return: java.util.Map<kd.hrmp.hbpm.business.utils.GroupKey, java.util.List < kd.bos.entity.ExtendedDataEntity>>
     * @author rd_ningjie
     * @date: 2022/9/19 14:36
     */
    public Map<GroupKey, List<ExtendedDataEntity>> buildKeys() {

        if (this.dataEntity == null || this.dataEntity.length <= 0 || propNames == null || propNames.size() <= 0) {
            return null;
        }
        Map<GroupKey, List<ExtendedDataEntity>> keyResult = new HashMap<>(this.dataEntity.length);
        DataEntityPropertyCollection props = this.dataEntity[0].getDataEntity().getDataEntityType().getProperties();
        for (int index = 0; index < this.dataEntity.length; index++) {
            GroupKey groupKey = buildKey(0, this.dataEntity[index].getDataEntity(), this.propNames, props);
            List<ExtendedDataEntity> entities = keyResult.get(groupKey);
            if (entities == null) {
                entities = new ArrayList<>();
            }
            entities.add(this.dataEntity[index]);
            keyResult.put(groupKey, entities);
        }
        return keyResult;
    }


    /**
     * @param dataIndex
     * @description: 场景  1、初始时从数据库带出来的数据（导入的时候没有填写，从数据库中查出来） 2、内存中校验的数据 3、查询出来的数据
     * @return: kd.hrmp.hbpm.business.utils.GroupKey
     * @author rd_ningjie
     * @date: 2022/9/16 11:20
     */
    public static GroupKey buildKey(int dataIndex, DynamicObject dy, List<String> propNames, DataEntityPropertyCollection props) {

        Object pkValue = dy.getPkValue();
        Object[] values = new Object[propNames.size()];
        Object[] valuesDesc = new Object[propNames.size()];

        for (int i = 0; i < propNames.size(); ++i) {
            String key = propNames.get(i);
            Object[] formatValues = formatValue(props.get(key), dy.get(key), dy);
            values[i] = formatValues[0];
            valuesDesc[i] = formatValues[1];
        }
        return new GroupKey(dataIndex, pkValue, values, valuesDesc);

    }

    /**
     * 对于一些特殊字段，它是有多个值，例如多语言，多选下拉框等等，即一个字段上有多个值，我们需要获取它的多个值的对象
     *
     * @param property 表单上控件的编码
     * @param value    值
     * @return
     */
    public static Object[] formatValue(IDataEntityProperty property, Object value, DynamicObject dy) {

        Object[] values = new Object[2];
        values[0] = value;
        values[1] = (value == null) ? ResManager.loadKDString("空值", GROUPKEYBUILDER_1, HRMP_HBJM_BUSINESS) : String.valueOf(value);

        if (property instanceof MuliLangTextProp) {
            // 多语言文本 //property.getValue(rowModel.getActiveRow())
            // 判断此字段是否在单据头
            boolean head = property.getParent() instanceof MainEntityType;
            DynamicObject data = null;
            if (head && !(dy.getDataEntityType() instanceof MainEntityType)) {
                data = (DynamicObject) dy.getParent();
            } else {
                data = dy;
            }
            if (property.getValueFast(data) instanceof OrmLocaleValue) {
                //直接设置ORMLOCALVALUE为对象无法判断相等
                values[0] = property.getValueFast(data);
            } else {
                values[0] = value;
            }
            values[1] = (value == null) ? ResManager.loadKDString("空值", GROUPKEYBUILDER_1, HRMP_HBJM_BUSINESS) : value.toString();
        }
        if (property instanceof BasedataProp) {
            // 基础资料字段，取内码比较值是否重复，取资料名称生成提示信息
            BasedataProp bdProp = (BasedataProp) property;
            if (value == null) {
                if (bdProp.getRefIdProp() instanceof LongProp) {
                    values[0] = 0L;
                } else {
                    values[0] = "";
                }
                values[1] = ResManager.loadKDString("空值", GROUPKEYBUILDER_1, HRMP_HBJM_BUSINESS);
            } else if (value instanceof DynamicObject) {
                values[0] = ((DynamicObject) value).getPkValue();
                values[1] = bdProp.getDisplayValue(value);
            }
        } else if (property instanceof ComboProp) {
            values[1] = ((ComboProp) property).getItemByName((String) value);
        } else if (property instanceof IntegerProp) {
            DecimalFormat format = new DecimalFormat("0");
            if (ObjectUtils.isEmpty(value)) {
                value = 0;
            }
            values[1] = format.format(value);
        } else if (property instanceof DecimalProp) {
            StringBuilder sBuilder = new StringBuilder();
            sBuilder.append(",###.");
            for (int i = 1; i <= ((DecimalProp) property).getScale(); i++) {
                sBuilder.append('0');
            }
            DecimalFormat format = new DecimalFormat(sBuilder.toString());
            if (ObjectUtils.isEmpty(value)) {
                value = 0;
            }
            values[1] = format.format(value);
        } else if (property instanceof DateProp) {
            if (value != null) {
                InteTimeZone zone = ((DateTimeProp) property).getTimeZone(0L);
                values[1] = zone.format((Date) value, new SimpleDateFormat("yyyy-MM-dd"));
            }
        } else if (property instanceof TimeProp) {
            if (value != null) {
                if (value instanceof Integer && value.equals(-1)) {
                    values[1] = ResManager.loadKDString("空值", GROUPKEYBUILDER_1, HRMP_HBJM_BUSINESS);
                } else {
                    if (value instanceof Integer) {
                        values[1] = getTimeFormat((Integer) value);
                    }
                }
            }
        } else if (property instanceof DateTimeProp) {
            if (value != null) {
                InteTimeZone zone = ((DateTimeProp) property).getTimeZone(0L);
                values[1] = zone.format((Date) value, new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            }
        } else if (property instanceof BooleanProp) {
            values[1] = (boolean) value ? ResManager.loadKDString("是", "GroupKeyBuilder_2", HRMP_HBJM_BUSINESS) : ResManager.loadKDString("否", "GroupKeyBuilder_3", HRMP_HBJM_BUSINESS);
        } else if (property instanceof TextProp) {
            if (StringUtils.isBlank(value)) {
                value = "";
                values[1] = StringUtils.isBlank(value) ? ResManager.loadKDString("空值", GROUPKEYBUILDER_1, HRMP_HBJM_BUSINESS) : value;
            }
        }
        return values;
    }


    public static String getTimeFormat(int time) {
        int temp = time;
        int hh = temp / 3600;
        int mm = (temp % 3600) / 60;
        int ss = (temp % 3600) % 60;
        return (hh < 10 ? ("0" + hh) : hh) + ":" +
                (mm < 10 ? ("0" + mm) : mm) + ":" +
                (ss < 10 ? ("0" + ss) : ss);
    }
}
